package de.meinefirma.meinprojekt.dao;

import java.util.List;
import de.meinefirma.meinprojekt.entities.Person;
import de.meinefirma.meinprojekt.entities.Team;

/** DAO fuer Person und Team */
public class PersonTeamDao extends AbstractDao
{
   // An HTML-Formular angepasste Methode fuer Zuordnung von Personen zu Teams:
   @SuppressWarnings("unchecked")
   public String addDelPersonTeam( String add, String del, String pname, String tname,
                                   Object personListe, Object teamListe )
   {
      Person person = null;
      Team   team   = null;

      if( (add == null && del == null) || pname == null || tname == null ||
            pname.trim().length() == 0 || tname.trim().length() == 0 ||
            !(personListe instanceof List) || !(teamListe instanceof List) )
         return "";

      // Damit Optmistic-Locking-Erkennung funktioniert, muessen die "alten" detached Objekte
      // verwendet werden (mit den alten "@Version"-Werten)
      // (personListe und teamListe wurden in der HttpSession zwischengespeichert):
      for( Person p : (List<Person>) personListe )
         if( p != null && p.getName() != null && p.getName().equals( pname ) ) {
            person = p;
            break;
         }
      for( Team t : (List<Team>) teamListe )
         if( t != null && t.getName() != null && t.getName().equals( tname ) ) {
            team = t;
            break;
         }
      if( person == null ) return "Person nicht vorhanden";
      if( team   == null ) return "Team nicht vorhanden";

      try {
         tx.begin();
         // Merge, damit Optmistic-Locking-Exceptions erkannt werden.
         // Der Returnwert beinhaltet die "managed instance", also die reattached Entity:
         person = em.merge( person );
         team   = em.merge( team );
         // Fuehre "del" bzw. "add" aus:
         if( del != null )
            team.removePerson( person );
         if( add != null ) {
            for( Team t : (List<Team>) teamListe )
               if( t != null && t.removePerson( person ) ) em.merge( t );
            team.addPerson( person );
         }
         tx.commit();
         return "";
      } catch( Exception ex ) {
         try { tx.rollback(); } catch( Exception e ) {/* nothing todo */}
         throw new RuntimeException( ex );
      }
   }
}
