package de.meinefirma.meinprojekt.dao;

import java.util.List;
import javax.inject.Inject;
import javax.naming.InitialContext;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.transaction.UserTransaction;
import org.jboss.arquillian.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.Archive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.EmptyAsset;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.junit.Assert;
import org.junit.Test;
import org.junit.runner.RunWith;
import de.meinefirma.meinprojekt.entities.*;

@RunWith( Arquillian.class )
public class PersonTeamDaoTest
{
   @PersistenceContext( unitName="MeineJpaPU", name="persistence/em" )
   EntityManager em;

   @Inject
   UserTransaction utx;

   @Deployment
   public static Archive<?> createDeployment()
   {
      return ShrinkWrap.create( WebArchive.class, "test.war" )
                       .addPackage( Person.class.getPackage() )
                       .addPackage( PersonTeamDao.class.getPackage() )
                       .addManifestResource( "test-persistence.xml", "persistence.xml" )
                       .addWebResource( EmptyAsset.INSTANCE, "beans.xml" );
   }

   @Test
   public void testEmTx() throws Exception
   {
      UserTransaction txJndi = (UserTransaction) (new InitialContext()).lookup( "java:comp/UserTransaction" );
      EntityManager   emJndi = (EntityManager)   (new InitialContext()).lookup( "java:comp/env/persistence/em" );
      Assert.assertNotNull( "UserTransaction", txJndi );
      Assert.assertNotNull( "EntityManager",   emJndi );
   }

   @Test
   public void testPersonTeamDao()
   {
      // Konstruiere das zu testende DAO:
      PersonTeamDao dao = new PersonTeamDao();

      // Loesche eventuell vorhandene Eintraege:
      dao.deleteAllEntities( Person.class );
      dao.deleteAllEntities( Team.class );

      // Teste createEntity() und queryEntities():
      Person p1 = new Person();
      Team   t1 = new Team();
      p1.setName( "p1" );
      t1.setName( "t1" );
      dao.createEntity( p1 );
      dao.createEntity( t1 );
      List<Person> p1Lst = dao.queryEntities( Person.QUERY_BY_NAME, "pname", "p1" );
      List<Team>   t1Lst = dao.queryEntities(   Team.QUERY_BY_NAME, "tname", "t1" );
      Assert.assertEquals(    1, p1Lst.size() );
      Assert.assertEquals(    1, t1Lst.size() );
      Assert.assertEquals( "p1", p1Lst.get( 0 ).getName() );
      Assert.assertEquals( "t1", t1Lst.get( 0 ).getName() );

      // Teste addDelPersonTeam() mit "add" (Zuordnung hinzufuegen):
      List<Person> pAll = dao.readAllEntities( Person.class );
      List<Team>   tAll = dao.readAllEntities( Team.class );
      dao.addDelPersonTeam( "add", null, "p1", "t1", pAll, tAll );
      pAll = dao.readAllEntities( Person.class );
      tAll = dao.readAllEntities( Team.class );
      Person p = pAll.get( 0 );
      Team   t = tAll.get( 0 );
      Assert.assertEquals( "t1", p.getTeam().getName() );
      Assert.assertTrue( t.getPersonen().contains( p ) );

      // Teste addDelPersonTeam() mit "del" (Zuordnung entfernen):
      dao.addDelPersonTeam( null, "del", "p1", "t1", pAll, tAll );
      pAll = dao.readAllEntities( Person.class );
      tAll = dao.readAllEntities( Team.class );
      p = pAll.get( 0 );
      t = tAll.get( 0 );
      Assert.assertNull( p.getTeam() );
      Assert.assertTrue( t.getPersonen().isEmpty() );

      System.out.println( "---- testPersonTeamDao() ok ----" );
   }
}
