package de.meinefirma.meinprojekt.entities;

import javax.persistence.*;
import java.io.Serializable;
import java.sql.Timestamp;

@Entity
@Table( uniqueConstraints=@UniqueConstraint(columnNames={"personName"}) )
@NamedQuery( name=Person.QUERY_BY_NAME, query="Select p from Person p where p.personName = :pname" )
public class Person implements Serializable
{
   private static final long   serialVersionUID  = 1L;
   public  static final String QUERY_BY_NAME     = "Person.queryByName";
   public  static final String HTML_TABLE_HEADER = "<tr><th>Id</th><th>LastUpdate</th><th>Name</th><th>Team.Name</th></tr>\n";

   @Id @GeneratedValue( strategy = GenerationType.AUTO )
   private Long id;

   @Version
   private Timestamp lastUpdate;

   @Column( nullable = false, length = 200 )
   private String personName;

   @ManyToOne( cascade=CascadeType.PERSIST )
   private Team team;

   // Getter/Setter:
   public Long      getId()         { return id;         }
   public Timestamp getLastUpdate() { return lastUpdate; }
   public String    getName()       { return personName; }
   public Team      getTeam()       { return team;       }
   public void setId(         Long      id         ) { this.id = id;                 }
   public void setLastUpdate( Timestamp lastUpdate ) { this.lastUpdate = lastUpdate; }
   public void setName(       String    name       ) { this.personName = name;       }
   public void setTeam(       Team      team       ) { this.team = team;             }

   @Override public boolean equals( Object obj ) {
      if( !(obj instanceof Person) ) return false;
      String onm = ((Person) obj).getName();
      return personName == onm || (personName != null && personName.equals( onm ));
   }

   @Override public int hashCode() {
      return ( personName == null ) ? 0 : personName.hashCode();
   }

   @Override public String toString() {
      return superToString( super.toString() ) +
             ": id=" + id + ", lastUpdate='" + lastUpdate + "', name='" + personName + "', team='" +
             ((team != null) ? team.getName() : "(kein Team zugeordnet)")+ "'";
   }

   public String toHtmlTableRow() {
      return "<tr><td>" + id + "</td><td>" + lastUpdate + "</td><td>" + personName + "</td><td>" +
             ((team != null) ? team.getName() : "(kein Team zugeordnet)") +
             "</td></tr>\n";
   }

   private static String superToString( String s ) {
      return s.substring( s.lastIndexOf( '.' ) + 1, s.length() );
   }
}
