package de.meinefirma.meinprojekt.servletfilter;

import javax.annotation.Resource;
import javax.naming.*;
import javax.persistence.*;
import javax.servlet.*;
import javax.transaction.UserTransaction;

@PersistenceContext( unitName="MeineJpaPU", name="persistence/em" )
public class JpaServletFilter implements Filter
{
   private static final ThreadLocal<EntityManager> entityManagerHolder = new ThreadLocal<>();

   @Resource private UserTransaction utx;

   @Override public void doFilter( ServletRequest requ, ServletResponse resp, FilterChain chain )
   throws ServletException
   {
      try {
         EntityManager em = (EntityManager) (new InitialContext()).lookup( "java:comp/env/persistence/em" );
         utx.begin();
         entityManagerHolder.set( em );
         chain.doFilter( requ, resp );
         utx.commit();
      } catch( Exception ex ) {
         try { utx.rollback(); } catch( Exception e ) {/*ok*/}
         throw new ServletException( ex );
      } finally {
         entityManagerHolder.remove();
      }
   }

   public static EntityManager getEntityManager()
   {
      return entityManagerHolder.get();
   }

   @Override public void init( FilterConfig arg0 ) { /* nicht benoetigt */ }
   @Override public void destroy() { /* nicht benoetigt */ }
}
